from airtouch2.protocol.messages.ResponseMessage import ResponseMessage
from airtouch2.protocol.messages.RequestState import RequestState
from airtouch2.protocol.messages.ac_commands import ChangeSetTemperature, ToggleAC, SetFanSpeed, SetMode
from airtouch2.protocol.messages.group_commands import ToggleGroup, ChangeDamper